@@ -34,7 +34,7 @@ class cyclic_reward_wrapper(RewardWrapper):
         self.curriculum_step = 0
         super().__init__(env)
 
-    def reset(self, observe=True):
+    def reset(self):
         self.reward_queues = {agent: queuesum() for agent in self.agents}
         for qs in self.reward_queues.values():
             qs.resize(self.curriculum[0][1])
