@@ -3,6 +3,7 @@ import setuptools
 with open("README.md", "r") as fh:
     long_description = fh.read()
 
+
 def get_version():
     path = "supersuit/__init__.py"
     with open(path) as file:
@@ -13,6 +14,7 @@ def get_version():
             return line.strip().split()[-1].strip().strip('"')
     raise RuntimeError("bad version data in __init__.py")
 
+
 setuptools.setup(
     name="SuperSuit",
     version=get_version(),
@@ -24,8 +26,8 @@ setuptools.setup(
     url="https://github.com/PettingZoo-Team/SuperSuit",
     keywords=["Reinforcement Learning", "gym"],
     packages=setuptools.find_packages(),
-    python_requires='>=3.6',
-    install_requires=["pettingzoo>=1.3.0", "opencv-python"],
+    python_requires=">=3.6",
+    install_requires=["pettingzoo>=1.4.0", "opencv-python"],
     classifiers=[
         "Programming Language :: Python :: 3.6",
         "Programming Language :: Python :: 3.7",
@@ -33,5 +35,6 @@ setuptools.setup(
         "License :: OSI Approved :: MIT License",
         "Operating System :: OS Independent",
     ],
+    extras={"dev": ["pettingzoo[butterfly]"]},
     include_package_data=True,
 )
