@@ -1,10 +1,9 @@
 from pettingzoo.utils.to_parallel import ParallelEnv
 import gym
-from gym.spaces import Box, Space, Discrete
+from gym.spaces import Box, Discrete
 from .adv_transforms.frame_stack import stack_obs_space, stack_init, stack_obs
 from .adv_transforms.frame_skip import check_transform_frameskip
 from .adv_transforms.obs_delay import Delayer
-import numpy as np
 
 
 class ParallelWraper(ParallelEnv):
@@ -12,14 +11,18 @@ class ParallelWraper(ParallelEnv):
         self.env = env
         self.observation_spaces = env.observation_spaces
         self.action_spaces = env.action_spaces
-        self.agents = env.agents
-        self.num_agents = env.num_agents
+        self.metadata = env.metadata
+        self.possible_agents = env.possible_agents
 
     def reset(self):
-        return self.env.reset()
+        res = self.env.reset()
+        self.agents = self.env.agents
+        return res
 
     def step(self, actions):
-        return self.env.step(actions)
+        res = self.env.step(actions)
+        self.agents = self.env.agents
+        return res
 
     def render(self, mode="human"):
         return self.env.render(mode)
@@ -27,18 +30,20 @@ class ParallelWraper(ParallelEnv):
     def close(self):
         return self.env.close()
 
+
 class ObservationWrapper(ParallelWraper):
     def reset(self):
-        obss = self.env.reset()
+        obss = super().reset()
         return {agent: self._modify_observation(agent, obs) for agent, obs in obss.items()}
 
     def step(self, actions):
-        obss, rew, done, info = self.env.step(actions)
+        obss, rew, done, info = super().step(actions)
         obss = {agent: self._modify_observation(agent, obs) for agent, obs in obss.items()}
         return obss, rew, done, info
 
+
 class frame_stack(ObservationWrapper):
-    def __init__(self,env,num_frames=4):
+    def __init__(self, env, num_frames=4):
         self.stack_size = num_frames
         super().__init__(env)
         self._check_wrapper_params()
@@ -52,7 +57,7 @@ class frame_stack(ObservationWrapper):
             elif isinstance(space, Discrete):
                 pass
             else:
-                assert False, "Stacking is currently only allowed for Box and Discrete observation spaces. The given observation space is {}".format(obs_space)
+                assert False, "Stacking is currently only allowed for Box and Discrete observation spaces. The given observation space is {}".format(space)
 
     def reset(self):
         self.stack = {agent: stack_init(space, self.stack_size) for agent, space in self.env.observation_spaces.items()}
@@ -63,6 +68,7 @@ class frame_stack(ObservationWrapper):
         self.stack[agent] = stack_obs(self.stack[agent], observation, space, self.stack_size)
         return self.stack[agent]
 
+
 class delay_observations(ObservationWrapper):
     def __init__(self, env, delay):
         super().__init__(env)
@@ -75,6 +81,7 @@ class delay_observations(ObservationWrapper):
         self.delayers = {agent: Delayer(space, self.delay) for agent, space in self.observation_spaces.items()}
         return super().reset()
 
+
 class frame_skip(ParallelWraper):
     def __init__(self, env, num_frames):
         super().__init__(env)
@@ -87,14 +94,23 @@ class frame_skip(ParallelWraper):
 
     def step(self, action):
         low, high = self.num_frames
-        num_skips = int(self.np_random.randint(low, high+1))
-        total_reward = {agent: 0. for agent in self.agents}
+        num_skips = int(self.np_random.randint(low, high + 1))
 
         for x in range(num_skips):
             obs, rews, done, info = super().step(action)
+            if x == 0:
+                next_agents = self.env.agents[:]
+                total_reward = {agent: 0.0 for agent in self.env.agents}
+                total_dones = {}
+                total_infos = {}
+                total_obs = {}
+
             for agent, rew in rews.items():
                 total_reward[agent] += rew
+                total_dones[agent] = done[agent]
+                total_infos[agent] = info[agent]
+                total_obs[agent] = obs[agent]
             if all(done.values()):
                 break
-
-        return obs, total_reward, done, info
+        self.agents = next_agents
+        return total_obs, total_reward, total_dones, total_infos
