@@ -1,12 +1,18 @@
 from .dummy_gym_env import DummyEnv
 from gym.spaces import Box, Discrete
 import numpy as np
-from supersuit import frame_stack_v1, reshape_v0, observation_lambda_v0, action_lambda_v0, dtype_v0
+from supersuit import (
+    frame_stack_v1,
+    reshape_v0,
+    observation_lambda_v0,
+    action_lambda_v0,
+    dtype_v0,
+)
 import supersuit
 import pytest
 
 base_obs = (np.zeros([8, 8, 3]) + np.arange(3)).astype(np.float32)
-base_obs_space = Box(low=np.float32(0.), high=np.float32(10.),shape=[8,8,3])
+base_obs_space = Box(low=np.float32(0.0), high=np.float32(10.0), shape=[8, 8, 3])
 base_act_spaces = Discrete(5)
 
 
@@ -20,7 +26,7 @@ def test_reshape():
 
 
 def new_continuous_dummy():
-    base_act_spaces = Box(low=np.float32(0.), high=np.float32(10.), shape=[3])
+    base_act_spaces = Box(low=np.float32(0.0), high=np.float32(10.0), shape=[3])
     return DummyEnv(base_obs, base_obs_space, base_act_spaces)
 
 
@@ -35,9 +41,9 @@ wrappers = [
     supersuit.dtype_v0(new_dummy(), np.int32),
     supersuit.flatten_v0(new_dummy()),
     supersuit.reshape_v0(new_dummy(), (64, 3)),
-    supersuit.normalize_obs_v0(new_dummy(), env_min=-1, env_max=5.),
+    supersuit.normalize_obs_v0(new_dummy(), env_min=-1, env_max=5.0),
     supersuit.frame_stack_v1(new_dummy(), 8),
-    supersuit.reward_lambda_v0(new_dummy(), lambda x: x/10),
+    supersuit.reward_lambda_v0(new_dummy(), lambda x: x / 10),
     supersuit.clip_reward_v0(new_dummy()),
     supersuit.clip_actions_v0(new_continuous_dummy()),
     supersuit.frame_skip_v0(new_dummy(), 4),
@@ -45,6 +51,8 @@ wrappers = [
     supersuit.sticky_actions_v0(new_dummy(), 0.75),
     supersuit.delay_observations_v0(new_dummy(), 1),
 ]
+
+
 @pytest.mark.parametrize("env", wrappers)
 def test_basic_wrappers(env):
     env.seed(5)
@@ -59,7 +67,8 @@ def test_basic_wrappers(env):
 
 def test_lambda():
     def add1(obs):
-        return obs+1
+        return obs + 1
+
     base_env = DummyEnv(base_obs, base_obs_space, base_act_spaces)
     env = observation_lambda_v0(base_env, add1)
     obs0 = env.reset()
@@ -70,7 +79,7 @@ def test_lambda():
 
     def tile_obs(obs):
         shape_size = len(obs.shape)
-        tile_shape = [1]*shape_size
+        tile_shape = [1] * shape_size
         tile_shape[0] *= 2
         return np.tile(obs, tile_shape)
 
@@ -80,6 +89,7 @@ def test_lambda():
 
     def change_shape_fn(obs_space):
         return Box(low=0, high=1, shape=(32, 8, 3))
+
     env = observation_lambda_v0(env, tile_obs)
     obs0 = env.reset()
     assert env.observation_space.shape == (32, 8, 3)
@@ -87,12 +97,12 @@ def test_lambda():
 
 
 def test_action_lambda():
-
     def inc1(x, space):
-        return (x + 1)
+        return x + 1
 
     def change_space_fn(space):
-        return Discrete(space.n+1)
+        return Discrete(space.n + 1)
+
     base_env = DummyEnv(base_obs, base_obs_space, base_act_spaces)
     env = action_lambda_v0(base_env, inc1, change_space_fn)
     env.reset()
@@ -105,16 +115,18 @@ def test_action_lambda():
 
     act_spaces = Box(low=0, high=1, shape=(15,))
     base_env = DummyEnv(base_obs, base_obs_space, act_spaces)
-    env = action_lambda_v0(base_env,
+    env = action_lambda_v0(
+        base_env,
         lambda action, act_space: one_hot(action, act_space.shape[0]),
-        lambda act_space: Discrete(act_space.shape[0]))
+        lambda act_space: Discrete(act_space.shape[0]),
+    )
 
     env.reset()
     env.step(2)
 
 
 def test_rew_lambda():
-    env = supersuit.reward_lambda_v0(new_dummy(), lambda x: x/10)
+    env = supersuit.reward_lambda_v0(new_dummy(), lambda x: x / 10)
     env.reset()
     obs, rew, done, info = env.step(0)
-    assert rew == 1./10
+    assert rew == 1.0 / 10
