@@ -201,9 +201,9 @@ class BallSprite(pygame.sprite.Sprite):
 
 class CooperativePong(gym.Env):
 
-    metadata = {'render.modes': ['human']}
+    metadata = {'render.modes': ['human', "rgb_array"]}
 
-    def __init__(self, randomizer, ball_speed=9, left_paddle_speed=12, right_paddle_speed=12, cake_paddle=True, max_frames=900, bounce_randomness=False):
+    def __init__(self, randomizer, ball_speed=9, left_paddle_speed=12, right_paddle_speed=12, cake_paddle=True, max_cycles=900, bounce_randomness=False):
         super(CooperativePong, self).__init__()
 
         pygame.init()
@@ -221,14 +221,12 @@ class CooperativePong(gym.Env):
         # self.observation_space = [gym.spaces.Box(low=0.0, high=1.0, shape=(original_shape), dtype=np.float32) for _ in range(self.num_agents)]
         self.observation_space = [gym.spaces.Box(low=0, high=255, shape=(original_color_shape), dtype=np.uint8) for _ in range(self.num_agents)]
 
-        self.clock = pygame.time.Clock()
-
         self.renderOn = False
 
         # set speed
         self.speed = [ball_speed, left_paddle_speed, right_paddle_speed]
 
-        self.max_frames = max_frames
+        self.max_cycles = max_cycles
 
         # paddles
         self.p0 = PaddleSprite((20 // RENDER_RATIO, 80 // RENDER_RATIO), left_paddle_speed)
@@ -285,12 +283,16 @@ class CooperativePong(gym.Env):
     def enable_render(self):
         self.screen = pygame.display.set_mode(self.screen.get_size())
         self.renderOn = True
+        self.draw()
 
     def render(self, mode='human'):
-        if not self.renderOn:
+        if not self.renderOn and mode == "human":
             # sets self.renderOn to true and initializes display
             self.enable_render()
+
+        observation = pygame.surfarray.pixels3d(self.screen)
         pygame.display.flip()
+        return np.transpose(observation, axes=(1, 0, 2)) if mode == "rgb_array" else None
 
     def observe(self, agent):
         observation = pygame.surfarray.pixels3d(self.screen)
@@ -319,6 +321,7 @@ class CooperativePong(gym.Env):
         # action: 0: do nothing,
         # action: 1: p[i] move up, 2: p[i] move down
         if agent == self.agents[0]:
+            self.rewards = {a: 0 for a in self.agents}
             self.p0.update(self.area, action)
         elif agent == self.agents[1]:
             self.p1.update(self.area, action)
@@ -338,17 +341,11 @@ class CooperativePong(gym.Env):
                 if not self.done:
                     self.num_frames += 1
                     # scaling reward so that the max reward is 100
-                    reward = 100 / self.max_frames
+                    reward = 100 / self.max_cycles
                     self.score += reward
-                    if self.num_frames == self.max_frames:
+                    if self.num_frames == self.max_cycles:
                         self.done = True
 
-                # let the clock tick
-                if self.renderOn:
-                    self.clock.tick(15)
-                else:
-                    self.clock.tick()
-
                 for ag in self.agents:
                     self.rewards[ag] = reward / self.num_agents
                     self.dones[ag] = self.done
@@ -372,7 +369,7 @@ parallel_env = parallel_wrapper_fn(env)
 
 class raw_env(AECEnv, EzPickle):
     # class env(MultiAgentEnv):
-    metadata = {'render.modes': ['human']}
+    metadata = {'render.modes': ['human', "rgb_array"]}
 
     def __init__(self, **kwargs):
         EzPickle.__init__(self, **kwargs)
@@ -380,8 +377,8 @@ class raw_env(AECEnv, EzPickle):
 
         self.seed()
 
-        self.agents = self.env.agents
-        self.num_agents = len(self.agents)
+        self.agents = self.env.agents[:]
+        self.possible_agents = self.agents[:]
         self._agent_selector = agent_selector(self.agents)
         self.agent_selection = self._agent_selector.reset()
         # spaces
@@ -402,14 +399,14 @@ class raw_env(AECEnv, EzPickle):
         self.randomizer, seed = seeding.np_random(seed)
         self.env = CooperativePong(self.randomizer, **self._kwargs)
 
-    def reset(self, observe=True):
+    def reset(self):
         self.env.reset()
+        self.agents = self.possible_agents[:]
         self.agent_selection = self._agent_selector.reset()
         self.rewards = self.env.rewards
+        self._cumulative_rewards = {a: 0 for a in self.agents}
         self.dones = self.env.dones
         self.infos = self.env.infos
-        if observe:
-            return self.observe(self.agent_selection)
 
     def observe(self, agent):
         obs = self.env.observe(agent)
@@ -419,9 +416,11 @@ class raw_env(AECEnv, EzPickle):
         self.env.close()
 
     def render(self, mode='human'):
-        self.env.render(mode)
+        return self.env.render(mode)
 
-    def step(self, action, observe=True):
+    def step(self, action):
+        if self.dones[self.agent_selection]:
+            return self._was_done_step(action)
         agent = self.agent_selection
         if np.isnan(action):
             action = 0
@@ -438,9 +437,9 @@ class raw_env(AECEnv, EzPickle):
 
         self.score = self.env.score
 
-        if observe:
-            observation = self.observe(self.agent_selection)
-            return observation
+        self._cumulative_rewards[agent] = 0
+        self._accumulate_rewards()
+        self._dones_step_first()
 
 # This was originally created, in full, by Ananth Hari in a different repo, and was
 # added in by Justin Terry (which is why he's shown as the creator in the git history)
