@@ -5,11 +5,12 @@ import pygame
 def manual_control(**kwargs):
     from .cooperative_pong import env as _env
     env = _env(**kwargs)
-    env.reset(observe=True)
+    env.reset()
 
     quit_loop = 0
 
     pygame.key.set_repeat(20, 0)
+    clock = pygame.time.Clock()
 
     total_reward = 0
     initial_iteration = {agent: True for agent in env.agents}
@@ -17,7 +18,7 @@ def manual_control(**kwargs):
     done = all(dones.values())
 
     while not done:
-
+        clock.tick(15)
         action_dict = {agent: 0 for agent in env.agents}  # do nothing
         for event in pygame.event.get():
             if event.type == pygame.QUIT:
@@ -48,10 +49,10 @@ def manual_control(**kwargs):
 
         for _ in env.agents:
             agent = env.agent_selection
-            reward, dones[agent], _ = env.last()
+            obs, reward, dones[agent], _ = env.last()
             total_reward += reward
             initial_iteration[agent] = False
-            env.step(action_dict[agent], observe=False)
+            env.step(action_dict[agent])
         done = all(env.dones.values())
 
         env.render()
