@@ -39,16 +39,15 @@ parallel_env = parallel_wrapper_fn(env)
 
 class raw_env(AECEnv, EzPickle):
 
-    metadata = {'render.modes': ['human']}
+    metadata = {'render.modes': ['human', "rgb_array"]}
 
-    def __init__(self, spawn_rate=20, num_archers=2, num_knights=2, killable_knights=True, killable_archers=True, pad_observation=True, black_death=True, line_death=False, max_frames=900):
-        EzPickle.__init__(self, spawn_rate, num_archers, num_knights, killable_knights, killable_archers, pad_observation, black_death, line_death, max_frames)
+    def __init__(self, spawn_rate=20, num_archers=2, num_knights=2, killable_knights=True, killable_archers=True, pad_observation=True, black_death=True, line_death=False, max_cycles=900):
+        EzPickle.__init__(self, spawn_rate, num_archers, num_knights, killable_knights, killable_archers, pad_observation, black_death, line_death, max_cycles)
         # Game Constants
         self.ZOMBIE_SPAWN = spawn_rate
-        self.FPS = 90
         self.WIDTH = 1280
         self.HEIGHT = 720
-        self.max_frames = 500
+        self.max_cycles = max_cycles
         self.frames = 0
         self.pad_observation = pad_observation
         self.killable_knights = killable_knights
@@ -65,7 +64,6 @@ class raw_env(AECEnv, EzPickle):
         self.sword_dict = {}
 
         # Game Variables
-        self.frame_count = 0
         self.score = 0
         self.run = True
         self.arrow_spawn_rate = self.sword_spawn_rate = self.zombie_spawn_rate = 0
@@ -95,7 +93,6 @@ class raw_env(AECEnv, EzPickle):
         # self.WINDOW = pygame.display.set_mode([self.WIDTH, self.HEIGHT])
         self.WINDOW = pygame.Surface((self.WIDTH, self.HEIGHT))
         pygame.display.set_caption("Knights, Archers, Zombies")
-        self.clock = pygame.time.Clock()
         self.left_wall = get_image(os.path.join('img', 'left_wall.png'))
         self.right_wall = get_image(os.path.join('img', 'right_wall.png'))
         self.right_wall_rect = self.right_wall.get_rect()
@@ -144,9 +141,9 @@ class raw_env(AECEnv, EzPickle):
         self.observation_spaces = dict(zip(self.agents, [Box(low=0, high=255, shape=(512, 512, 3), dtype=np.uint8) for _ in enumerate(self.agents)]))
         self.action_spaces = dict(zip(self.agents, [Discrete(6) for _ in enumerate(self.agents)]))
         self.display_wait = 0.0
+        self.possible_agents = self.agents[:]
 
         self._agent_selector = agent_selector(self.agents)
-        self.num_agents = len(self.agents)
         self.reinit()
 
     def seed(self, seed=None):
@@ -397,12 +394,11 @@ class raw_env(AECEnv, EzPickle):
 
         return np.swapaxes(cropped, 1, 0)
 
-    def step(self, action, observe=True):
+    def step(self, action):
+        if self.dones[self.agent_selection]:
+            return self._was_done_step(action)
         agent = self.agent_selection
-        if self.render_on:
-            self.clock.tick(self.FPS)                # FPS
-        else:
-            self.clock.tick()
+
         if self._agent_selector.is_last():
             # Controls the Spawn Rate of Weapons
             self.sword_spawn_rate, self.arrow_spawn_rate = self.check_weapon_spawn(self.sword_spawn_rate, self.arrow_spawn_rate)
@@ -469,52 +465,58 @@ class raw_env(AECEnv, EzPickle):
                 self.arrow_list.remove(arrow)
                 self.all_sprites.remove(arrow)
 
-            self.WINDOW.fill((66, 40, 53))
-            self.WINDOW.blit(self.left_wall, self.left_wall.get_rect())
-            self.WINDOW.blit(self.right_wall, self.right_wall_rect)
-            self.WINDOW.blit(self.floor_patch1, (500, 500))
-            self.WINDOW.blit(self.floor_patch2, (900, 30))
-            self.WINDOW.blit(self.floor_patch3, (150, 430))
-            self.WINDOW.blit(self.floor_patch4, (300, 50))
-            self.WINDOW.blit(self.floor_patch1, (1000, 250))
-            self.all_sprites.draw(self.WINDOW)       # Draw all the sprites
+            self.draw()
 
             self.check_game_end()
             self.frames += 1
 
+        self._clear_rewards()
         self.rewards[agent] = agent_name.score
-        done = not self.run or self.frames >= self.max_frames
-        self.dones = {agent: done for agent in self.agents}
+        done = not self.run or self.frames >= self.max_cycles
+        self.dones = {a: done for a in self.agents}
 
         if self._agent_selector.is_last() and not self.black_death:
+            _live_agents = self.agents[:]
             # self.agents must be recreated
             for k in self.kill_list:
-                self.agents.remove(k)
-                # self.dones.pop(k, None)
-                # self.rewards.pop(k, None)
-                # self.infos.pop(k, None)
-
-            self._agent_selector.reinit(self.agents)
-            self.num_agents = len(self.agents)
+                self.dones[k] = True
+                _live_agents.remove(k)
 
             # reset the kill list
             self.kill_list = []
 
+            self._agent_selector.reinit(_live_agents)
+
         self.agent_selection = self._agent_selector.next()
-        if observe:
-            return self.observe(self.agent_selection)
+        self._cumulative_rewards[agent] = 0
+        self._accumulate_rewards()
+        self._dones_step_first()
 
     def enable_render(self):
         self.WINDOW = pygame.display.set_mode([self.WIDTH, self.HEIGHT])
         # self.WINDOW = pygame.Surface((self.WIDTH, self.HEIGHT))
         self.render_on = True
-        self.reset()
+        self.draw()
+
+    def draw(self):
+        self.WINDOW.fill((66, 40, 53))
+        self.WINDOW.blit(self.left_wall, self.left_wall.get_rect())
+        self.WINDOW.blit(self.right_wall, self.right_wall_rect)
+        self.WINDOW.blit(self.floor_patch1, (500, 500))
+        self.WINDOW.blit(self.floor_patch2, (900, 30))
+        self.WINDOW.blit(self.floor_patch3, (150, 430))
+        self.WINDOW.blit(self.floor_patch4, (300, 50))
+        self.WINDOW.blit(self.floor_patch1, (1000, 250))
+        self.all_sprites.draw(self.WINDOW)       # Draw all the sprites
 
     def render(self, mode="human"):
-        if not self.render_on:
+        if not self.render_on and mode == "human":
             # sets self.render_on to true and initializes display
             self.enable_render()
+
+        observation = np.array(pygame.surfarray.pixels3d(self.WINDOW))
         pygame.display.flip()
+        return np.transpose(observation, axes=(1, 0, 2)) if mode == "rgb_array" else None
 
     def close(self):
         if not self.closed:
@@ -533,11 +535,6 @@ class raw_env(AECEnv, EzPickle):
         # Zombie Kills all Players
         self.run = self.zombie_all_players(self.knight_list, self.archer_list, self.run)
 
-        # Condition to Check 900 Frames
-        self.frame_count += 1
-        if self.frame_count > 900:
-            self.run = False
-
     def reinit(self):
         # Dictionaries for holding new players and their weapons
         self.archer_dict = {}
@@ -546,7 +543,6 @@ class raw_env(AECEnv, EzPickle):
         self.sword_dict = {}
 
         # Game Variables
-        self.frame_count = 0
         self.score = 0
         self.run = True
         self.arrow_spawn_rate = self.sword_spawn_rate = self.zombie_spawn_rate = 0
@@ -599,18 +595,19 @@ class raw_env(AECEnv, EzPickle):
             self.agent_name_mapping[k_name] = a_count
             a_count += 1
 
+        self.draw()
         self.frames = 0
 
-    def reset(self, observe=True):
+    def reset(self):
         self.has_reset = True
+        self.agents = self.possible_agents[:]
         self._agent_selector.reinit(self.agents)
         self.agent_selection = self._agent_selector.next()
         self.rewards = dict(zip(self.agents, [0 for _ in self.agents]))
+        self._cumulative_rewards = {a: 0 for a in self.agents}
         self.dones = dict(zip(self.agents, [False for _ in self.agents]))
         self.infos = dict(zip(self.agents, [{} for _ in self.agents]))
         self.reinit()
-        if observe:
-            return self.observe(self.agent_selection)
 
 # The original code for this game, that was added by Justin Terry, was
 # created by Dipam Patel in a different repository (hence the git history)
