@@ -56,10 +56,12 @@ class Prospector(pg.sprite.Sprite):
 
     def reset(self, pos):
         self.body.angle = 0
+        self.body.angular_velocity = 0
         self.image = pg.transform.rotozoom(self.orig_image, 0, 1)
         self.rect = self.image.get_rect(center=pos)
         self.body.position = utils.flipy(pos)
         self.body.velocity = Vec2d(0.0, 0.0)
+        self.body.force = Vec2d(0.0, 0.0)
         self.body.nugget = None
 
     @property
@@ -114,7 +116,6 @@ class Banker(pg.sprite.Sprite):
         self.body = pm.Body(1, moment, body_type=pm.Body.DYNAMIC)
         self.body.nugget = None
         self.body.sprite_type = "banker"
-        self.body.nugget_offset = None
 
         self.shape = pm.Circle(self.body, const.AGENT_RADIUS)
         self.shape.collision_type = CollisionTypes.BANKER
@@ -451,7 +452,7 @@ class raw_env(AECEnv, EzPickle):
         prospec_handoff_gold_reward=1,
         banker_receive_gold_reward=1,
         banker_deposit_gold_reward=1,
-        max_frames=900,
+        max_cycles=900,
     ):
         EzPickle.__init__(
             self,
@@ -462,7 +463,7 @@ class raw_env(AECEnv, EzPickle):
             prospec_handoff_gold_reward,
             banker_receive_gold_reward,
             banker_deposit_gold_reward,
-            max_frames,
+            max_cycles,
         )
 
         total_reward_factor = ind_reward + group_reward + other_group_reward
@@ -472,7 +473,6 @@ class raw_env(AECEnv, EzPickle):
                 "group reward should add up to approximately 1.0"
             )
 
-        self.num_agents = const.NUM_AGENTS
         self.agents = []
 
         self.sprite_list = [
@@ -481,11 +481,10 @@ class raw_env(AECEnv, EzPickle):
             "bankers/2.png",
             "prospector.png",
         ]
-        self.max_frames = max_frames
+        self.max_cycles = max_cycles
 
         pg.init()
         self.seed()
-        self.clock = pg.time.Clock()
         self.closed = False
 
         self.background = Background(self.rng)
@@ -533,7 +532,7 @@ class raw_env(AECEnv, EzPickle):
             f = Fence(w_type, s_pos, b_pos, verts, self.space)
             self.fences.append(f)
 
-        self.metadata = {"render.modes": ["human"]}
+        self.metadata = {"render.modes": ["human", "rgb_array"]}
 
         self.action_spaces = {}
         for p in self.prospectors:
@@ -561,6 +560,7 @@ class raw_env(AECEnv, EzPickle):
                 low=0, high=255, shape=const.BANKER_OBSERV_SHAPE, dtype=np.uint8
             )
 
+        self.possible_agents = self.agents[:]
         self._agent_selector = agent_selector(self.agents)
         self.agent_selection = self._agent_selector.next()
         self.reset()
@@ -624,7 +624,6 @@ class raw_env(AECEnv, EzPickle):
                 gold_sprite.parent_body.nugget = None
                 gold_sprite.parent_body = banker_body
                 banker_body.nugget = gold_sprite
-                banker_body.nugget_offset = normal.angle
 
                 for k, v in self.prospectors.items():
                     self.rewards[k] += other_group_reward * banker_receive_gold_reward
@@ -739,11 +738,12 @@ class raw_env(AECEnv, EzPickle):
 
         return sub_screen
 
-    def step(self, action, observe=True):
+    def step(self, action):
+        if self.dones[self.agent_selection]:
+            return self._was_done_step(action)
         agent_id = self.agent_selection
         all_agents_updated = self._agent_selector.is_last()
-        if all_agents_updated:
-            self.rewards = {agent: 0 for agent in self.agents}
+        self.rewards = {agent: 0 for agent in self.agents}
 
         if agent_id in self.prospectors:
             agent = self.prospectors[agent_id]
@@ -760,11 +760,6 @@ class raw_env(AECEnv, EzPickle):
 
         # Only take next step in game if all agents have received an action
         if all_agents_updated:
-            if self.rendering:
-                self.clock.tick(const.FPS)
-            else:
-                self.clock.tick()
-
             for _ in range(const.STEPS_PER_FRAME):
                 self.space.step(const.SPACE_STEP_DELTA)
 
@@ -790,21 +785,18 @@ class raw_env(AECEnv, EzPickle):
 
             self.frame += 1
             # If we reached max frames, we're done
-            if self.frame == self.max_frames:
+            if self.frame == self.max_cycles:
                 self.dones = dict(zip(self.agents, [True for _ in self.agents]))
 
         if self.rendering:
             pg.event.pump()
 
         self.agent_selection = self._agent_selector.next()
+        self._cumulative_rewards[agent_id] = 0
+        self._accumulate_rewards()
+        self._dones_step_first()
 
-        if observe:
-            return self.observe(self.agent_selection)
-
-    def reward(self):
-        return self.rewards
-
-    def reset(self, observe=True):
+    def reset(self):
         self.screen = pg.Surface(const.SCREEN_SIZE)
         self.done = False
 
@@ -823,35 +815,39 @@ class raw_env(AECEnv, EzPickle):
 
         self.gold = []
 
+        self.agents = self.possible_agents[:]
         self.rewards = dict(zip(self.agents, [0 for _ in self.agents]))
+        self._cumulative_rewards = dict(zip(self.agents, [0 for _ in self.agents]))
         self.dones = dict(zip(self.agents, [False for _ in self.agents]))
         self.infos = dict(zip(self.agents, [{} for _ in self.agents]))
-        self.metadata = {"render.modes": ["human"]}
         self.rendering = False
         self.frame = 0
-        self.dirty_rects = []
         self.background.dirty_rects.clear()
 
         self._agent_selector.reinit(self.agents)
         self.agent_selection = self._agent_selector.next()
         self.full_draw()
-        if observe:
-            return self.observe(self.agent_selection)
 
     def render(self, mode="human"):
-        if not self.rendering:
-            pg.display.init()
-            self.screen = pg.display.set_mode(const.SCREEN_SIZE)
-            self.background.convert_img()
-            self.water.convert_img()
-            for f in self.fences:
-                f.convert_img()
-            for s in self.all_sprites.sprites():
-                s.convert_img()
-            self.rendering = True
-            self.full_draw()
+        if mode == "human":
+            if not self.rendering:
+                self.rendering = True
+                pg.display.init()
+                self.screen = pg.display.set_mode(const.SCREEN_SIZE)
+                self.background.convert_img()
+                self.water.convert_img()
+                for f in self.fences:
+                    f.convert_img()
+                for s in self.all_sprites.sprites():
+                    s.convert_img()
+                self.full_draw()
 
-        pg.display.flip()
+            pg.display.flip()
+
+        elif mode == "rgb_array":  # no display, return whole screen as array
+            observation = np.array(pg.surfarray.pixels3d(self.screen))
+            transposed = np.transpose(observation, axes=(1, 0, 2))
+            return transposed
 
     def full_draw(self):
         """ Called to draw everything when first rendering """
