@@ -10,6 +10,7 @@ from pettingzoo.utils import wrappers
 
 def env():
     env = raw_env()
+    env = wrappers.CaptureStdoutWrapper(env)
     env = wrappers.TerminateIllegalWrapper(env, illegal_reward=-1)
     env = wrappers.AssertOutOfBoundsWrapper(env)
     env = wrappers.NaNRandomWrapper(env)
@@ -27,6 +28,7 @@ class raw_env(AECEnv):
         self.board = chess.Board()
 
         self.agents = ["player_{}".format(i) for i in range(2)]
+        self.possible_agents = self.agents[:]
 
         self._agent_selector = agent_selector(self.agents)
 
@@ -39,31 +41,28 @@ class raw_env(AECEnv):
 
         self.agent_selection = None
 
-        self.num_agents = len(self.agents)
-
     def seed(self, seed=None):
         pass
 
     def observe(self, agent):
-        return chess_utils.get_observation(self.board, self.agents.index(agent))
+        return chess_utils.get_observation(self.board, self.possible_agents.index(agent))
 
-    def reset(self, observe=True):
+    def reset(self):
         self.has_reset = True
 
+        self.agents = self.possible_agents[:]
+
         self.board = chess.Board()
 
+        self._agent_selector = agent_selector(self.agents)
         self.agent_selection = self._agent_selector.reset()
 
         self.rewards = {name: 0 for name in self.agents}
+        self._cumulative_rewards = {name: 0 for name in self.agents}
         self.dones = {name: False for name in self.agents}
         self.infos = {name: {'legal_moves': []} for name in self.agents}
         self.infos[self.agent_selection]['legal_moves'] = chess_utils.legal_moves(self.board)
 
-        if observe:
-            return self.observe(self.agent_selection)
-        else:
-            return
-
     def set_game_result(self, result_val):
         for i, name in enumerate(self.agents):
             self.dones[name] = True
@@ -71,7 +70,9 @@ class raw_env(AECEnv):
             self.rewards[name] = result_val * result_coef
             self.infos[name] = {'legal_moves': []}
 
-    def step(self, action, observe=True):
+    def step(self, action):
+        if self.dones[self.agent_selection]:
+            return self._was_done_step(action)
         current_agent = self.agent_selection
         current_index = self.agents.index(current_agent)
         self.agent_selection = next_agent = self._agent_selector.next()
@@ -99,14 +100,12 @@ class raw_env(AECEnv):
             self.infos[next_agent] = {'legal_moves': next_legal_moves}
             assert len(self.infos[next_agent]['legal_moves'])
 
-        if observe:
-            next_observation = self.observe(next_agent)
-        else:
-            next_observation = None
-        return next_observation
+        self._accumulate_rewards()
+        self._dones_step_first()
 
     def render(self, mode='human'):
         print(self.board)
+        return str(self.board)
 
     def close(self):
         pass
