@@ -10,6 +10,7 @@ from pettingzoo.utils import wrappers
 def env(**kwargs):
     env = raw_env(**kwargs)
     pass_move = env._N * env._N
+    env = wrappers.CaptureStdoutWrapper(env)
     env = wrappers.TerminateIllegalWrapper(env, illegal_reward=-1)
     env = wrappers.AssertOutOfBoundsWrapper(env)
     env = wrappers.NanNoOpWrapper(env, pass_move, "passing turn with action {}".format(pass_move))
@@ -30,7 +31,7 @@ class raw_env(AECEnv):
         self._komi = komi
 
         self.agents = ['black_0', 'white_0']
-        self.num_agents = len(self.agents)
+        self.possible_agents = self.agents[:]
         self.has_reset = False
 
         self.observation_spaces = self._convert_to_dict([spaces.Box(low=0, high=1, shape=(self._N, self._N, 3), dtype=np.bool) for _ in range(self.num_agents)])
@@ -54,13 +55,13 @@ class raw_env(AECEnv):
         return 0 <= c[0] < self._N and 0 <= c[1] < self._N
 
     def _encode_player_plane(self, agent):
-        if agent == self.agents[0]:
+        if agent == self.possible_agents[0]:
             return np.zeros([self._N, self._N], dtype=np.bool)
         else:
             return np.ones([self._N, self._N], dtype=np.bool)
 
     def _encode_board_planes(self, agent):
-        agent_factor = -1 if agent == self.agents[0] else 1
+        agent_factor = -1 if agent == self.possible_agents[0] else 1
         current_agent_plane_idx = np.where(self._go.board == agent_factor)
         opponent_agent_plane_idx = np.where(self._go.board == -agent_factor)
         current_agent_plane = np.zeros([self._N, self._N], dtype=np.bool)
@@ -70,13 +71,13 @@ class raw_env(AECEnv):
         return current_agent_plane, opponent_agent_plane
 
     def _int_to_name(self, ind):
-        return self.agents[ind]
+        return self.possible_agents[ind]
 
     def _name_to_int(self, name):
-        return self.agents.index(name)
+        return self.possible_agents.index(name)
 
     def _convert_to_dict(self, list_of_list):
-        return dict(zip(self.agents, list_of_list))
+        return dict(zip(self.possible_agents, list_of_list))
 
     def _encode_legal_actions(self, actions):
         return np.where(actions == 1)[0]
@@ -89,7 +90,9 @@ class raw_env(AECEnv):
         player_plane = self._encode_player_plane(agent)
         return np.dstack((current_agent_plane, opponent_agent_plane, player_plane))
 
-    def step(self, action, observe=True):
+    def step(self, action):
+        if self.dones[self.agent_selection]:
+            return self._was_done_step(action)
         self._go = self._go.play_move(coords.from_flat(action))
         self._last_obs = self.observe(self.agent_selection)
         next_player = self._agent_selector.next()
@@ -100,23 +103,22 @@ class raw_env(AECEnv):
         else:
             self.infos[next_player]['legal_moves'] = self._encode_legal_actions(self._go.all_legal_moves())
         self.agent_selection = next_player if next_player else self._agent_selector.next()
-        if observe:
-            return self._last_obs
+        self._accumulate_rewards()
+        self._dones_step_first()
 
-    def reset(self, observe=True):
+    def reset(self):
         self.has_reset = True
         self._go = go.Position(board=None, komi=self._komi)
 
+        self.agents = self.possible_agents[:]
+        self._agent_selector.reinit(self.agents)
         self.agent_selection = self._agent_selector.reset()
+        self._cumulative_rewards = self._convert_to_dict(np.array([0.0, 0.0]))
         self.rewards = self._convert_to_dict(np.array([0.0, 0.0]))
         self.dones = self._convert_to_dict([False for _ in range(self.num_agents)])
         self.infos = self._convert_to_dict([{'legal_moves': []} for _ in range(self.num_agents)])
         self.infos[self.agent_selection]['legal_moves'] = self._encode_legal_actions(self._go.all_legal_moves())
         self._last_obs = self.observe(self.agents[0])
-        if observe:
-            return self._last_obs
-        else:
-            return
 
     def render(self, mode='human'):
         print(self._go)
