@@ -25,6 +25,7 @@ Wrapper class around Deepmind's Hanabi Learning Environment.
 def env(**kwargs):
     env = raw_env(**kwargs)
     player_losing_reward = -3
+    env = wrappers.CaptureStdoutWrapper(env)
     env = wrappers.TerminateIllegalWrapper(env, illegal_reward=player_losing_reward)
     env = wrappers.AssertOutOfBoundsWrapper(env)
     env = wrappers.NaNRandomWrapper(env)
@@ -46,7 +47,6 @@ class raw_env(AECEnv, EzPickle):
         'max_information_tokens',
         'max_life_tokens',
         'observation_type',
-        'seed',
         'random_start_player',
     }
 
@@ -54,11 +54,10 @@ class raw_env(AECEnv, EzPickle):
                  colors: int = 5,
                  ranks: int = 5,
                  players: int = 2,
-                 hand_size: int = 2,
+                 hand_size: int = 5,
                  max_information_tokens: int = 8,
                  max_life_tokens: int = 3,
                  observation_type: int = 1,
-                 seed=None,
                  random_start_player: bool = False,
                  ):
 
@@ -73,7 +72,6 @@ class raw_env(AECEnv, EzPickle):
               - observation_type: int.
                     0: Minimal observation.
                     1: First-order common knowledge observation.
-              - seed: int, Random seed or None.
               - random_start_player: bool, Random start player.
 
         Common game configurations:
@@ -83,24 +81,27 @@ class raw_env(AECEnv, EzPickle):
                 "players": 2,
                 "max_information_tokens": 8,
                 "max_life_tokens": 3,
+                "hand_size": (4 if players >= 4 else 5)
                 "observation_type": 1,
                 "hand_size": 2
                 }
 
             Hanabi-Small : {
-                "colors": 5,
-                "ranks": 5,
-                "players": 2,
-                "max_information_tokens":
-                "max_life_tokens":
-                "observation_type": 1}
-
-            Hanabi-Very-Small : {
                 "colors": 2,
                 "ranks": 5,
                 "players": 2,
-                "max_information_tokens":
-                "max_life_tokens":
+                "max_information_tokens": 3
+                "hand_size": 2,
+                "max_life_tokens": 1
+                "observation_type": 1}
+
+            Hanabi-Very-Small : {
+                "colors": 1,
+                "ranks": 5,
+                "players": 2,
+                "max_information_tokens": 3
+                "hand_size": 2,
+                "max_life_tokens": 1
                 "observation_type": 1}
 
         """
@@ -113,12 +114,9 @@ class raw_env(AECEnv, EzPickle):
             max_information_tokens,
             max_life_tokens,
             observation_type,
-            seed,
             random_start_player,
         )
 
-        seed = seeding.create_seed(seed, max_bytes=3)
-
         # ToDo: Starts
         # Check if all possible dictionary values are within a certain ranges.
         self._raise_error_if_config_values_out_of_range(colors,
@@ -144,11 +142,12 @@ class raw_env(AECEnv, EzPickle):
 
         # List of agent names
         self.agents = ["player_{}".format(i) for i in range(self.hanabi_env.players)]
+        self.possible_agents = self.agents[:]
 
         self.agent_selection: str
 
         # Sets hanabi game to clean state and updates all internal dictionaries
-        self.reset(observe=False)
+        self.reset()
 
         # Set action_spaces and observation_spaces based on params in hanabi_env
         self.action_spaces = {name: spaces.Discrete(self.hanabi_env.num_moves()) for name in self.agents}
@@ -195,10 +194,6 @@ class raw_env(AECEnv, EzPickle):
     def observation_vector_dim(self):
         return self.hanabi_env.vectorized_observation_shape()
 
-    @property
-    def num_agents(self):
-        return len(self.agents)
-
     @property
     def legal_moves(self) -> List[int]:
         return self.infos[self.agent_selection]['legal_moves']
@@ -208,7 +203,7 @@ class raw_env(AECEnv, EzPickle):
         return list(range(0, self.hanabi_env.num_moves()))
 
     # ToDo: Fix Return value
-    def reset(self, observe=True) -> Optional[List[int]]:
+    def reset(self):
         """ Resets the environment for a new game and returns observations of current player as List of ints
 
         Returns:
@@ -216,6 +211,7 @@ class raw_env(AECEnv, EzPickle):
             current agent (agent_selection).
         """
 
+        self.agents = self.possible_agents[:]
         # Reset underlying hanabi reinforcement learning environment
         obs = self.hanabi_env.reset()
 
@@ -223,15 +219,10 @@ class raw_env(AECEnv, EzPickle):
         self._reset_agents(player_number=obs['current_player'])
 
         self.rewards = {agent: 0 for agent in self.agents}
+        self._cumulative_rewards = {name: 0 for name in self.agents}
         # Reset internal state
         self._process_latest_observations(obs=obs)
 
-        # If specified, return observation of current agent
-        if observe:
-            return self.observe(agent_name=self.agent_selection)
-        else:
-            return None
-
     def _reset_agents(self, player_number: int):
         """ Rearrange self.agents as pyhanabi starts a different player after each reset(). """
 
@@ -257,6 +248,8 @@ class raw_env(AECEnv, EzPickle):
             By default a list of integers, describing the logic state of the game from the view of the agent.
             Can be a returned as a descriptive dictionary, if as_vector=False.
         """
+        if self.dones[self.agent_selection]:
+            return self._was_done_step(action)
         action = int(action)
 
         agent_on_turn = self.agent_selection
@@ -271,25 +264,22 @@ class raw_env(AECEnv, EzPickle):
             # Apply action
             all_observations, reward, done, _ = self.hanabi_env.step(action=action)
 
-            # sets current reward for 0 to intialize reward accumulation
-            self.rewards[agent_on_turn] = 0
-
             # Update internal state
             self._process_latest_observations(obs=all_observations, reward=reward, done=done)
 
-            # Return latest observations if specified
-            if observe:
-                return self.observe(agent_name=agent_on_turn)
+            # sets current reward for 0 to intialize reward accumulation
+            self._cumulative_rewards[agent_on_turn] = 0
+            self._accumulate_rewards()
+            self._dones_step_first()
 
     def observe(self, agent_name: str):
-        return np.array(self.infos[agent_name]['observations_vectorized'], np.float32)
+        return np.array(self.infos[agent_name]['observations_vectorized'], np.float32) if agent_name in self.infos else np.zeros_like(self.observation_spaces[agent_name].low)
 
     def _process_latest_observations(self, obs: Dict, reward: Optional[float] = 0, done: Optional[bool] = False):
         """Updates internal state"""
 
         self.latest_observations = obs
-        for agent, agent_rew in self.rewards.items():
-            self.rewards[agent] = reward + agent_rew
+        self.rewards = {a: reward for a in self.agents}
         self.dones = {player_name: done for player_name in self.agents}
 
         # Here we have to deal with the player index with offset = 1
@@ -307,9 +297,9 @@ class raw_env(AECEnv, EzPickle):
          Example:
         """
         player_data = self.latest_observations['player_observations']
-        print("Active player:", self.agents[player_data[0]['current_player_offset']])
+        print("Active player:", self.possible_agents[player_data[0]['current_player_offset']])
         for i, d in enumerate(player_data):
-            print(self.agents[i])
+            print(self.possible_agents[i])
             print("========")
             print(d['pyhanabi'])
             print()
