@@ -16,6 +16,7 @@ from gym.utils import EzPickle
 
 def env(**kwargs):
     env = raw_env(**kwargs)
+    env = wrappers.CaptureStdoutWrapper(env)
     env = wrappers.TerminateIllegalWrapper(env, illegal_reward=-1)
     env = wrappers.AssertOutOfBoundsWrapper(env)
     env = wrappers.NaNRandomWrapper(env)
@@ -27,9 +28,11 @@ class raw_env(RLCardBase, EzPickle):
 
     metadata = {'render.modes': ['human']}
 
-    def __init__(self, seed=None, knock_reward: float = 0.5, gin_reward: float = 1.0):
-        EzPickle.__init__(self, seed, knock_reward, gin_reward)
-        RLCardBase.__init__(self, "gin-rummy", 2, (5, 52), seed)
+    def __init__(self, knock_reward: float = 0.5, gin_reward: float = 1.0, opponents_hand_visible=False):
+        EzPickle.__init__(self, knock_reward, gin_reward)
+        self._opponents_hand_visible = opponents_hand_visible
+        num_planes = 5 if self._opponents_hand_visible else 4
+        RLCardBase.__init__(self, "gin-rummy", 2, (num_planes, 52))
         self._knock_reward = knock_reward
         self._gin_reward = gin_reward
 
@@ -50,8 +53,15 @@ class raw_env(RLCardBase, EzPickle):
             payoff = -deadwood_count / 100
         return payoff
 
+    def observe(self, agent):
+        obs = self.env.get_state(self._name_to_int(agent))
+        if self._opponents_hand_visible:
+            return obs['obs'].astype(self._dtype)
+        else:
+            return obs['obs'][0:4, :].astype(self._dtype)
+
     def render(self, mode='human'):
-        for player in self.agents:
+        for player in self.possible_agents:
             state = self.env.game.round.players[self._name_to_int(player)].hand
             print("\n===== {}'s Hand =====".format(player))
             print_card([c.__str__()[::-1] for c in state])
