@@ -11,6 +11,7 @@ from .rlcard_base import RLCardBase
 
 def env(**kwargs):
     env = raw_env(**kwargs)
+    env = wrappers.CaptureStdoutWrapper(env)
     env = wrappers.TerminateIllegalWrapper(env, illegal_reward=-1)
     env = wrappers.AssertOutOfBoundsWrapper(env)
     pass_action = 3
@@ -23,11 +24,11 @@ class raw_env(RLCardBase):
 
     metadata = {'render.modes': ['human']}
 
-    def __init__(self, seed=None):
-        super().__init__("limit-holdem", 2, (72,), seed)
+    def __init__(self):
+        super().__init__("limit-holdem", 2, (72,))
 
     def render(self, mode='human'):
-        for player in self.agents:
+        for player in self.possible_agents:
             state = self.env.game.get_state(self._name_to_int(player))
             print("\n=============== {}'s Hand ===============".format(player))
             print_card(state['hand'])
