@@ -11,6 +11,7 @@ from .board import Board
 
 def env():
     env = raw_env()
+    env = wrappers.CaptureStdoutWrapper(env)
     env = wrappers.TerminateIllegalWrapper(env, illegal_reward=-1)
     env = wrappers.AssertOutOfBoundsWrapper(env)
     env = wrappers.NaNRandomWrapper(env)
@@ -25,8 +26,8 @@ class raw_env(AECEnv):
         super().__init__()
         self.board = Board()
 
-        self.num_agents = 2
         self.agents = ["player_1", "player_2"]
+        self.possible_agents = self.agents[:]
 
         self.action_spaces = {i: spaces.Discrete(9) for i in self.agents}
         self.observation_spaces = {i: spaces.Box(low=0, high=1, shape=(3, 3, 2), dtype=np.int8) for i in self.agents}
@@ -50,15 +51,20 @@ class raw_env(AECEnv):
     #  [2,1,0]]
     def observe(self, agent):
         board_vals = np.array(self.board.squares).reshape(3, 3)
-        cur_player = self.agents.index(self.agent_selection)
+        cur_player = self.possible_agents.index(self.agent_selection)
         opp_player = (cur_player + 1) % 2
 
         cur_p_board = np.equal(board_vals, cur_player + 1)
         opp_p_board = np.equal(board_vals, opp_player + 1)
         return np.stack([cur_p_board, opp_p_board], axis=2).astype(np.int8)
 
+    def seed(self, seed=None):
+        pass
+
     # action in this case is a value from 0 to 8 indicating position to move on tictactoe board
-    def step(self, action, observe=True):
+    def step(self, action):
+        if self.dones[self.agent_selection]:
+            return self._was_done_step(action)
         # check if input action is a valid move (0 == empty spot)
         assert (self.board.squares[action] == 0), "played illegal move"
         # play turn
@@ -90,28 +96,26 @@ class raw_env(AECEnv):
             self.dones = {i: True for i in self.agents}
 
         # Switch selection to next agents
+        self._cumulative_rewards[self.agent_selection] = 0
         self.agent_selection = next_agent
 
-        if observe:
-            return self.observe(self.agent_selection)
-        else:
-            return
+        self._accumulate_rewards()
+        self._dones_step_first()
 
-    def reset(self, observe=True):
+    def reset(self):
         # reset environment
         self.board = Board()
 
+        self.agents = self.possible_agents[:]
         self.rewards = {i: 0 for i in self.agents}
+        self._cumulative_rewards = {i: 0 for i in self.agents}
         self.dones = {i: False for i in self.agents}
         self.infos = {i: {'legal_moves': list(range(0, 9))} for i in self.agents}
 
         # selects the first agent
+        self._agent_selector.reinit(self.agents)
         self._agent_selector.reset()
         self.agent_selection = self._agent_selector.reset()
-        if observe:
-            return self.observe(self.agent_selection)
-        else:
-            return
 
     def render(self, mode='human'):
         def getSymbol(input):
