@@ -23,17 +23,16 @@ parallel_env = parallel_wrapper_fn(env)
 
 class raw_env(AECEnv, EzPickle):
 
-    metadata = {'render.modes': ['human']}
+    metadata = {'render.modes': ['human', "rgb_array"]}
 
     def __init__(self, *args, **kwargs):
         EzPickle.__init__(self, *args, **kwargs)
         self.env = _env(*args, **kwargs)
         pygame.init()
-        self.num_agents = self.env.num_agents
-        self.agents = ["pursuer_" + str(a) for a in range(self.num_agents)]
+        self.agents = ["pursuer_" + str(a) for a in range(self.env.num_agents)]
+        self.possible_agents = self.agents[:]
         self.agent_name_mapping = dict(zip(self.agents, list(range(self.num_agents))))
         self._agent_selector = agent_selector(self.agents)
-        self.has_reset = False
         # spaces
         self.n_act_agents = self.env.act_dims[0]
         self.action_spaces = dict(zip(self.agents, self.env.action_space))
@@ -46,42 +45,44 @@ class raw_env(AECEnv, EzPickle):
     def seed(self, seed=None):
         self.env.seed(seed)
 
-    def reset(self, observe=True):
-        self.has_reset = True
+    def reset(self):
         self.steps = 0
-        self.rewards = dict(
-            zip(self.agents, [np.float64(0) for _ in self.agents]))
+        self.agents = self.possible_agents[:]
+        self.rewards = dict(zip(self.agents, [(0) for _ in self.agents]))
+        self._cumulative_rewards = dict(zip(self.agents, [(0) for _ in self.agents]))
         self.dones = dict(zip(self.agents, [False for _ in self.agents]))
         self.infos = dict(zip(self.agents, [{} for _ in self.agents]))
         self._agent_selector.reinit(self.agents)
         self.agent_selection = self._agent_selector.next()
         self.env.reset()
-        if observe:
-            return self.observe(self.agent_selection)
 
     def close(self):
-        if not self.closed and self.has_reset:
+        if not self.closed:
             self.closed = True
             self.env.close()
 
     def render(self, mode="human"):
         if not self.closed:
-            self.env.render()
+            return self.env.render(mode)
 
-    def step(self, action, observe=True):
+    def step(self, action):
+        if self.dones[self.agent_selection]:
+            return self._was_done_step(action)
         agent = self.agent_selection
         self.env.step(action, self.agent_name_mapping[agent], self._agent_selector.is_last())
         for k in self.dones:
-            if self.env.frames >= self.env.max_frames:
+            if self.env.frames >= self.env.max_cycles:
                 self.dones[k] = True
             else:
                 self.dones[k] = self.env.is_terminal
         for k in self.agents:
             self.rewards[k] = self.env.latest_reward_state[self.agent_name_mapping[k]]
         self.steps += 1
+
+        self._cumulative_rewards[self.agent_selection] = 0
         self.agent_selection = self._agent_selector.next()
-        if observe:
-            return self.observe(self.agent_selection)
+        self._accumulate_rewards()
+        self._dones_step_first()
 
     def observe(self, agent):
         o = self.env.safely_observe(self.agent_name_mapping[agent])
