@@ -33,7 +33,7 @@ class Pursuit():
 
         tag_reward: reward for 'tagging' a single evader
 
-        max_frames: after how many frames should the game end
+        max_cycles: after how many frames should the game end
         n_catch: how surrounded evader needs to be, before removal
         freeze_evaders: toggle evaders move or not
         catch_reward: reward for pursuer who catches an evader
@@ -47,7 +47,7 @@ class Pursuit():
         x_size = self.x_size
         y_size = self.y_size
         self.map_matrix = two_d_maps.rectangle_map(self.x_size, self.y_size)
-        self.max_frames = kwargs.pop("max_frames", 500)
+        self.max_cycles = kwargs.pop("max_cycles", 500)
         self.seed()
 
         self.local_ratio = kwargs.pop('local_ratio', 1.0)
@@ -123,7 +123,6 @@ class Pursuit():
         self.renderOn = False
         self.pixel_scale = 30
 
-        self.clock = pygame.time.Clock()
         self.frames = 0
         self.reset()
         assert not kwargs, f"gave arguments {list(kwargs.keys())} that are not valid pursuit arguments"
@@ -194,12 +193,11 @@ class Pursuit():
         opponent_layer = self.evader_layer
         opponent_controller = self.evader_controller
 
-        if is_last:
-            self.latest_reward_state = self.reward()
-
         # actual action application
         agent_layer.move_agent(agent_id, action)
 
+        self.latest_reward_state = self.reward() / self.num_agents
+
         if is_last:
             ev_remove, pr_remove, pursuers_who_remove = self.remove_agents()
 
@@ -219,13 +217,7 @@ class Pursuit():
             global_val = self.latest_reward_state.mean()
             local_val = self.latest_reward_state
             self.latest_reward_state = self.local_ratio * local_val + (1 - self.local_ratio) * global_val
-
-        if self.renderOn:
-            self.clock.tick(15)
-        else:
-            self.clock.tick(2000)
-
-        self.frames = self.frames + 1
+            self.frames = self.frames + 1
 
     def draw_model_state(self):
         # -1 is building pixel flag
@@ -267,12 +259,12 @@ class Pursuit():
 
             pygame.draw.circle(self.screen, col, center, int(self.pixel_scale / 3))
 
-    def render(self):
-        if not self.renderOn:
+    def render(self, mode="human"):
+        if not self.renderOn and mode == "human":
             pygame.display.init()
             self.screen = pygame.display.set_mode(
                 (self.pixel_scale * self.x_size, self.pixel_scale * self.y_size))
-        self.renderOn = True
+            self.renderOn = True
         self.draw_model_state()
 
         self.draw_pursuers_observations()
@@ -280,7 +272,11 @@ class Pursuit():
         self.draw_evaders()
         self.draw_pursuers()
 
+        observation = pygame.surfarray.pixels3d(self.screen)
+        new_observation = np.copy(observation)
+        del observation
         pygame.display.flip()
+        return np.transpose(new_observation, axes=(1, 0, 2)) if mode == "rgb_array" else None
 
     def animate(self, act_fn, nsteps, file_name, rate=1.5, verbose=False):
         """
