@@ -75,7 +75,7 @@ class MAWaterWorld():
                  obstacle_radius=0.2, obstacle_loc=np.array([0.5, 0.5]), ev_speed=0.01,
                  poison_speed=0.01, n_sensors=30, sensor_range=0.2, action_scale=0.01,
                  poison_reward=-1., food_reward=10., encounter_reward=.01, control_penalty=-.5,
-                 local_ratio=1.0, speed_features=True, max_frames=500, **kwargs):
+                 local_ratio=1.0, speed_features=True, max_cycles=500, **kwargs):
         """
             n_pursuers: number of pursuing archea
             n_evaders: number of evaading archea
@@ -95,7 +95,7 @@ class MAWaterWorld():
             control_penalty: reward added to pursuer in each step
             local_ratio: proportion of reward allocated locally vs distributed among all agents
             speed_features: toggles whether archea sensors detect speed of other objects
-            max_frames: number of frames before environment automatically ends
+            max_cycles: number of frames before environment automatically ends
         """
         self.n_pursuers = n_pursuers
         self.n_evaders = n_evaders
@@ -121,7 +121,7 @@ class MAWaterWorld():
         self.n_obstacles = 1
         self.local_ratio = local_ratio
         self._speed_features = speed_features
-        self.max_frames = max_frames
+        self.max_cycles = max_cycles
         self.seed()
         self._pursuers = [
             Archea(npu + 1, self.radius, self.n_sensors, self.sensor_range[npu],
@@ -152,10 +152,6 @@ class MAWaterWorld():
             cv2.destroyAllWindows()
             cv2.waitKey(1)
 
-    @property
-    def timestep_limit(self):
-        return 1000
-
     @property
     def agents(self):
         return self._pursuers
@@ -173,7 +169,6 @@ class MAWaterWorld():
         return objx_2
 
     def reset(self):
-        self._timesteps = 0
         self.frames = 0
         # Initialize obstacles
         if self.obstacle_loc is None:
@@ -215,12 +210,6 @@ class MAWaterWorld():
 
         return obs_list[0]
 
-    @property
-    def is_terminal(self):
-        if self._timesteps >= self.timestep_limit:
-            return True
-        return False
-
     def _caught(self, is_colliding_N1_N2, n_coop):
         """ Checke whether collision results in catching the object
 
@@ -540,18 +529,15 @@ class MAWaterWorld():
             self.last_rewards = local_reward * self.local_ratio + global_reward * (1 - self.local_ratio)
 
             self.control_rewards = [0 for _ in range(self.n_pursuers)]
+            self.frames += 1
 
-        self.dones = [self.is_terminal for _ in range(self.n_pursuers)]
-
-        self._timesteps += 1
-        self.frames += 1
         return self.observe(agent_id)
 
     def observe(self, agent):
         return np.array(self.last_obs[agent], dtype=np.float32)
 
-    def render(self, screen_size=900, rate=5, mode='human'):
-        if not self.renderOn:
+    def render(self, mode='human', screen_size=900, rate=5):
+        if not self.renderOn and mode == "human":
             self.renderOn = True
             cv2.startWindowThread()
         img = np.empty((screen_size, screen_size, 3), dtype=np.uint8)
@@ -592,4 +578,4 @@ class MAWaterWorld():
         cv2.addWeighted(bg, opacity, img, 1 - opacity, 0, img)
         cv2.imshow('Waterworld', img)
         cv2.waitKey(rate)
-        return np.asarray(img)[..., ::-1]
+        return np.asarray(img)[..., ::-1] if mode == "rgb_array" else None
