@@ -6,13 +6,17 @@ from copy import copy
 def bombardment_test(env, cycles=10000):
     print("Starting bombardment test")
 
-    prev_observe = env.reset()
+    env.reset()
+    prev_observe, _, _, _ = env.last()
     observation_0 = copy(prev_observe)
     for i in range(cycles):
         if i == cycles / 2:
             print("\t50% through bombardment test")
         for agent in env.agent_iter(env.num_agents):  # step through every agent once with observe=True
-            if 'legal_moves' in env.infos[agent]:
+            obs, reward, done, info = env.last()
+            if done:
+                action = None
+            elif 'legal_moves' in env.infos[agent]:
                 action = random.choice(env.infos[agent]['legal_moves'])
             else:
                 action = env.action_spaces[agent].sample()
@@ -20,5 +24,6 @@ def bombardment_test(env, cycles=10000):
             assert env.observation_spaces[agent].contains(prev_observe), "Agent's observation is outside of its observation space"
             test_observation(prev_observe, observation_0)
             prev_observe = next_observe
-        prev_observe = env.reset()
+        env.reset()
+        prev_observe, _, _, _ = env.last()
     print("Passed bombardment test")
