@@ -6,23 +6,25 @@ def performance_benchmark(env):
     print("Starting performance benchmark")
     cycles = 0
     turn = 0
-    _ = env.reset()
+    env.reset()
     start = time.time()
     end = 0
 
     while True:
         cycles += 1
         for agent in env.agent_iter(env.num_agents):  # step through every agent once with observe=True
-            reward, done, info = env.last()
-            if not done and 'legal_moves' in env.infos[agent]:
+            obs, reward, done, info = env.last()
+            if done:
+                action = None
+            elif 'legal_moves' in env.infos[agent]:
                 action = random.choice(env.infos[agent]['legal_moves'])
             else:
                 action = env.action_spaces[agent].sample()
-            _ = env.step(action)
+            env.step(action)
             turn += 1
 
             if all(env.dones.values()):
-                _ = env.reset()
+                env.reset()
 
         if time.time() - start > 5:
             end = time.time()
