@@ -2,10 +2,12 @@ import os
 
 
 # add more dir names here
-dir_names = ["gamma", "sisl", "magent", "mpe"]
+dir_names = ["butterfly", "sisl", "magent", "mpe"]
+
+had_error = False
 
 for name in dir_names:
-    root_dir = os.path.join(os.pardir, name)
+    root_dir = os.path.join("pettingzoo", name)
 
     for _dir, subdirs, files in os.walk(root_dir):
         for file in files:
@@ -14,4 +16,7 @@ for name in dir_names:
                     for line in f:
                         if line.lstrip().startswith("print"):
                             print("File: {} has a print statement. Please remove it.".format(os.path.join(_dir, file)))
+                            had_error = True
                             break
+
+exit(-1 if had_error else 0)
