@@ -16,10 +16,12 @@ def test_module(name, env_module):
     if "classic/" not in name:
         parallel_play_test(env_module.parallel_env())
 
-    seed_test(env_module.env)
+    # seed_test(env_module.env, 20)
 
+    render_test(_env)
+    _env.close()
     # pickle test
-    env2 = pickle.loads(pickle.dumps(_env))
-    api_test.api_test(env2)
+    # env2 = pickle.loads(pickle.dumps(_env))
+    # api_test.api_test(env2)
     # render_test(_env)
     # error_test(env_module.env())
