@@ -6,6 +6,7 @@ import warnings
 from pettingzoo import AECEnv
 
 from .env_logger import EnvLogger
+from .capture_stdout import capture_stdout
 
 
 class BaseWrapper(AECEnv):
@@ -13,16 +14,15 @@ class BaseWrapper(AECEnv):
     Creates a wrapper around `env` parameter. Extend this class
     to create a useful wrapper.
     '''
-    metadata = {'render.modes': ['human']}
 
     def __init__(self, env):
         super().__init__()
         self.env = env
 
-        self.num_agents = self.env.num_agents
-        self.agents = self.env.agents
         self.observation_spaces = self.env.observation_spaces
         self.action_spaces = self.env.action_spaces
+        self.possible_agents = self.env.possible_agents
+        self.metadata = self.env.metadata
 
         # we don't want these defined as we don't want them used before they are gotten
 
@@ -44,107 +44,48 @@ class BaseWrapper(AECEnv):
         self.env.close()
 
     def render(self, mode='human'):
-        self.env.render(mode)
+        return self.env.render(mode)
 
-    def reset(self, observe=True):
-        observation = self.env.reset(observe)
+    def reset(self):
+        self.env.reset()
 
         self.agent_selection = self.env.agent_selection
         self.rewards = self.env.rewards
         self.dones = self.env.dones
         self.infos = self.env.infos
-
-        return observation
+        self.agents = self.env.agents
+        self._cumulative_rewards = self.env._cumulative_rewards
 
     def observe(self, agent):
         return self.env.observe(agent)
 
-    def step(self, action, observe=True):
-        next_obs = self.env.step(action, observe=observe)
+    def step(self, action):
+        self.env.step(action)
 
         self.agent_selection = self.env.agent_selection
         self.rewards = self.env.rewards
         self.dones = self.env.dones
         self.infos = self.env.infos
+        self.agents = self.env.agents
+        self._cumulative_rewards = self.env._cumulative_rewards
 
-        return next_obs
-
-
-class AgentIterWrapper(BaseWrapper):
-    def __init__(self, env):
-        super().__init__(env)
-        self._has_updated = False
-        self._is_iterating = False
-        self._agent_idxs = {agent: i for i, agent in enumerate(env.agents)}
-        # self._was_dones = {agent: False for agent in self.agents}
-
-    def reset(self, observe=True):
-        self._has_updated = True
-        self._final_rewards = {agent: 0 for agent in self.agents}
-        self.old_observation = None
-        obs = super().reset(observe)
-        self._was_dones = {agent: done for agent, done in self.dones.items()}
-        return obs
-
-    def last(self):
-        agent = self.agent_selection
-        return self.rewards[agent], self.dones[agent], self.infos[agent]
-
-    def step(self, action, observe=True):
-        self._has_updated = True
-
-        cur_agent = self.agent_selection
-        if self.agent_selection != self.env.agent_selection:
-            self._was_dones[cur_agent] = True
-            self.agent_selection = self.env.agent_selection
-        else:
-            self._was_dones[cur_agent] = self.env.dones[cur_agent]
-            super().step(action, False)
-
-        for agent in self.agents:
-            if self.dones[agent] and not self._final_rewards[agent]:
-                self._final_rewards[agent] = self.rewards[agent]
-
-        for agent in self.agents:
-            if self.dones[agent] and not self._was_dones[agent]:
-                self.agent_selection = agent
-                self.rewards[agent] = self._final_rewards[agent]
-                break
-
-        return super().observe(self.agent_selection) if observe else None
-
-    def agent_iter(self, max_iter=2**63):
-        return AECIterable(self, max_iter)
-
-
-class AECIterable:
-    def __init__(self, env, max_iter):
-        self.env = env
-        self.max_iter = max_iter
-
-    def __iter__(self):
-        return AECOrderEnforcingIterator(self.env, self.max_iter)
-
-
-class AECIterator:
-    def __init__(self, env, max_iter):
-        self.env = env
-        self.iters_til_term = max_iter
-        self.env._is_iterating = True
-
-    def __next__(self):
-        if self.env._was_dones[self.env.agent_selection] or self.iters_til_term <= 0:
-            raise StopIteration
-        self.iters_til_term -= 1
-        return self.env.agent_selection
-
-
-class AECOrderEnforcingIterator(AECIterator):
-    def __next__(self):
-        agent = super().__next__()
-        assert self.env._has_updated, "need to call step() or reset() in a loop over `agent_iter`!"
-        self.env._has_updated = False
-        return agent
+# class LastWrapper(BaseWrapper):
+#     def reset(self):
+#         super().reset()
+#         self._cumulative_rewards = dict(**self.rewards)
+#
+#     def step(self, action):
+#         agent = self.agent_selection
+#         super().step(action)
+#
+#         self._cumulative_rewards[agent] = 0
+#         for a, reward in self.rewards.items():
+#             self._cumulative_rewards[agent] += reward
+#
+#     def last(self, observe=True):
+#         agent = self.agent_selection
+#         observation = self.observe(agent) if observe else None
+#         return observation, self._cumulative_rewards[agent], self.dones[agent], self.infos[agent]
 
 
 class TerminateIllegalWrapper(BaseWrapper):
@@ -159,18 +100,27 @@ class TerminateIllegalWrapper(BaseWrapper):
         super().__init__(env)
         self._illegal_value = illegal_reward
 
-    def step(self, action, observe=True):
+    def reset(self):
+        self._terminated = False
+        super().reset()
+
+    def step(self, action):
         current_agent = self.agent_selection
         assert 'legal_moves' in self.infos[current_agent], "Illegal moves must always be defined to use the TerminateIllegalWrapper"
-        if action not in self.infos[current_agent]['legal_moves']:
+        if self._terminated and self.dones[self.agent_selection]:
+            self._was_done_step(action)
+        elif not self.dones[self.agent_selection] and action not in self.infos[current_agent]['legal_moves']:
             EnvLogger.warn_on_illegal_move()
             self.dones = {d: True for d in self.dones}
             for info in self.infos.values():
                 info['legal_moves'] = []
             self.rewards = {d: 0 for d in self.dones}
             self.rewards[current_agent] = self._illegal_value
+            self._dones_step_first()
+            self._accumulate_rewards()
+            self._terminated = True
         else:
-            return super().step(action, observe)
+            super().step(action)
 
 
 class NanNoOpWrapper(BaseWrapper):
@@ -183,11 +133,11 @@ class NanNoOpWrapper(BaseWrapper):
         self._no_op_action = no_op_action
         self._no_op_policy = no_op_policy
 
-    def step(self, action, observe=True):
-        if np.isnan(action).any():
+    def step(self, action):
+        if not (action is None and self.dones[self.agent_selection]) and np.isnan(action).any():
             EnvLogger.warn_action_is_NaN(self._no_op_policy)
             action = self._no_op_action
-        return super().step(action, observe)
+        super().step(action)
 
 
 class NanZerosWrapper(BaseWrapper):
@@ -199,11 +149,11 @@ class NanZerosWrapper(BaseWrapper):
         super().__init__(env)
         assert all(isinstance(space, Box) for space in self.action_spaces.values()), "should only use NanZerosWrapper for Box spaces. Use NanNoOpWrapper for discrete spaces"
 
-    def step(self, action, observe=True):
-        if np.isnan(action).any():
+    def step(self, action):
+        if not (action is None and self.dones[self.agent_selection]) and np.isnan(action).any():
             EnvLogger.warn_action_is_NaN("taking the all zeros action")
             action = np.zeros_like(action)
-        return super().step(action, observe)
+        super().step(action)
 
 
 class NaNRandomWrapper(BaseWrapper):
@@ -216,8 +166,8 @@ class NaNRandomWrapper(BaseWrapper):
         SEED = 0x33bb9cc9
         self.np_random = np.random.RandomState(SEED)
 
-    def step(self, action, observe=True):
-        if np.isnan(action).any():
+    def step(self, action):
+        if not (action is None and self.dones[self.agent_selection]) and np.isnan(action).any():
             cur_info = self.infos[self.agent_selection]
             if 'legal_moves' in cur_info:
                 backup_policy = "taking a random legal action"
@@ -229,7 +179,24 @@ class NaNRandomWrapper(BaseWrapper):
                 act_space = self.action_spaces[self.agent_selection]
                 action = self.np_random.choice(act_space.n)
 
-        return super().step(action, observe)
+        super().step(action)
+
+
+class CaptureStdoutWrapper(BaseWrapper):
+    def __init__(self, env):
+        super().__init__(env)
+        self.metadata['render.modes'].append("ansi")
+
+    def render(self, mode="human"):
+        if mode == "human":
+            super().render()
+        elif mode == "ansi":
+            with capture_stdout() as stdout:
+
+                super().render()
+
+                val = stdout.getvalue()
+            return val
 
 
 class AssertOutOfBoundsWrapper(BaseWrapper):
@@ -241,9 +208,9 @@ class AssertOutOfBoundsWrapper(BaseWrapper):
         super().__init__(env)
         assert all(isinstance(space, Discrete) for space in self.action_spaces.values()), "should only use AssertOutOfBoundsWrapper for Discrete spaces"
 
-    def step(self, action, observe=True):
-        assert self.action_spaces[self.agent_selection].contains(action), "action is not in action space"
-        return super().step(action, observe)
+    def step(self, action):
+        assert (action is None and self.dones[self.agent_selection]) or self.action_spaces[self.agent_selection].contains(action), "action is not in action space"
+        super().step(action)
 
 
 class ClipOutOfBoundsWrapper(BaseWrapper):
@@ -254,18 +221,18 @@ class ClipOutOfBoundsWrapper(BaseWrapper):
         super().__init__(env)
         assert all(isinstance(space, Box) for space in self.action_spaces.values()), "should only use ClipOutOfBoundsWrapper for Box spaces"
 
-    def step(self, action, observe=True):
+    def step(self, action):
         space = self.action_spaces[self.agent_selection]
-        if not space.contains(action):
+        if not (action is None and self.dones[self.agent_selection]) and not space.contains(action):
             assert space.shape == action.shape, "action should have shape {}, has shape {}".format(space.shape, action.shape)
 
             EnvLogger.warn_action_out_of_bound(action=action, action_space=space, backup_policy="clipping to space")
             action = np.clip(action, space.low, space.high)
 
-        return super().step(action, observe)
+        super().step(action)
 
 
-class OrderEnforcingWrapper(AgentIterWrapper):
+class OrderEnforcingWrapper(BaseWrapper):
     '''
     check all orders:
 
@@ -286,7 +253,7 @@ class OrderEnforcingWrapper(AgentIterWrapper):
         '''
         if value == "agent_order":
             raise AttributeError("agent_order has been removed from the API. Please consider using agent_iter instead.")
-        elif value in {"rewards", "dones", "infos", "agent_selection"}:
+        elif value in {"rewards", "dones", "infos", "agent_selection", "num_agents", "agents"}:
             raise AttributeError("{} cannot be accessed before reset".format(value))
         else:
             raise AttributeError("'{}' object has no attribute '{}'".format(type(self).__name__, value))
@@ -298,8 +265,9 @@ class OrderEnforcingWrapper(AgentIterWrapper):
     def render(self, mode='human'):
         if not self._has_reset:
             EnvLogger.error_render_before_reset()
+        assert mode in self.metadata['render.modes']
         self._has_rendered = True
-        super().render(mode)
+        return super().render(mode)
 
     def close(self):
         super().close()
@@ -311,22 +279,20 @@ class OrderEnforcingWrapper(AgentIterWrapper):
         self._has_rendered = False
         self._has_reset = False
 
-    def step(self, action, observe=True):
+    def step(self, action):
         if not self._has_reset:
             EnvLogger.error_step_before_reset()
-        elif self._was_dones[self.agent_selection]:
+        elif not self.agents:
             EnvLogger.warn_step_after_done()
-            self.dones = {agent: True for agent in self.dones}
-            self.rewards = {agent: 0 for agent in self.rewards}
-            return super().observe(self.agent_selection) if observe else None
+            return None
         else:
-            return super().step(action, observe)
+            super().step(action)
 
     def observe(self, agent):
         if not self._has_reset:
             EnvLogger.error_observe_before_reset()
         return super().observe(agent)
 
-    def reset(self, observe=True):
+    def reset(self):
         self._has_reset = True
-        return super().reset(observe)
+        super().reset()
